/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.util;

import java.util.Stack;
import javax.microedition.lcdui.List;

/**
 * A multithreaded queue used for 
 * implementing producer-consumer style threading patterns.
 * Multiple threads can wait for runnable objects being added
 * to the queue while other threads add to the queue.
 */
public class Queue {

    private Stack tasks = new Stack();
    private long defaultTimeout = 100;

    public Queue() {
    }

    /**
     * Returns the current number of runnable objects in the queue
     */
    public synchronized int size() {
        return tasks.size();
    }

    /** 
     * adds a runnable object to the end of the queue.
     * At least one thread will be notified.
     */
    public synchronized void add(Object runnable) {
        tasks.push(runnable);
        notifyAll();
    }

    /** 
     * Removes the first runnable object from the queue, blocking until one is available.
     */
    public synchronized Object remove() {
        while (true) {
            Object runnable = removeNoWait();
            if ( runnable != null ) {
                return runnable;
            }
            try {
                wait( defaultTimeout );
            }
            catch (InterruptedException e) {
                e.printStackTrace();
                Log.error(this, "InterruptedException  in remove() method");
            }
        }
    }

    /**
     * Removes the first runnable object from the queue without blocking.
     * This method will return immediately with an item from the queue or null.
     * @return the first runnable object removed from the queue or null if the
     * queue is empty
     */
    public synchronized Object removeNoWait() {
        if ( ! tasks.empty() ) {
            return tasks.pop();
        } else {
            return null;
        }
    }

}